// mini calendar
// Copyright (c) 1999 Hisashi HORIKAWA. All rights reserved.

#include "../config.h"

#include <cstdio>
#include <string>
#include <sys/types.h>
#include <sys/stat.h>     // mkdir()
#include <dirent.h>       // readdir()
#include <time.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

extern "C" {
    #include "callbacks.h"
    #include "interface.h"
}
#include "cls.h"
#include "support.h"
#include <misc.h>
#include <misc-gtk.h>
#include <dialog.h>

using namespace std;

////////////////////////////////////////////////////////////////////////

#define APP_NAME "mini calendar"
#define APP_VERSION VERSION

CalenWindow calenWindow;
ItemListWindow itemListWindow;

typedef ptr_set<EditWindow*, less<EditWindow*> > EditWins;
EditWins editWins;

EditWindow* findEditWindow(GtkWidget* widget)
{
  widget = getWindow(widget);
  EditWins::const_iterator i;
  for (i = editWins.begin(); i != editWins.end(); i++) {
    if ((*i)->window == widget)
      return *i;
  }
  return NULL;
}

EditWindow* findEditWindow(int y, int m, int d)
{
  EditWins::const_iterator i;
  for (i = editWins.begin(); i != editWins.end(); i++) {
    if ((*i)->cur_year == y && (*i)->cur_month == m && (*i)->cur_day == d)
      return *i;
  }
  return NULL;
}

string getPathRoot()
{
  return string(getenv("HOME")) + "/calendar";
}

string getPathName(int y, int m)
{
    char buf[1000];
    sprintf(buf, "%s/%04d%02d", getPathRoot().c_str(), y, m);
    return buf;
}


////////////////////////////////////////////////////////////////////////
// EditWindow

EditWindow::EditWindow(int y, int m, int d):
              cur_year(y), cur_month(m), cur_day(d)
{
}

EditWindow::~EditWindow()
{
}

string EditWindow::getCurFile() const
{
    char buf[1000];
    sprintf(buf, "%s/%02d.txt",
                getPathName(cur_year, cur_month).c_str(), cur_day);
    return buf;
}

void EditWindow::save()
{
    gtk_text_freeze(GTK_TEXT(text));
    char* const buf = gtk_editable_get_chars(GTK_EDITABLE(text), 0, -1);
    for (const char* p = buf; *p; p++) {
        if (!isspace(*p)) {
            mkdir(getPathRoot().c_str(), S_IRWXU);
            mkdir(getPathName(cur_year, cur_month).c_str(), S_IRWXU);

            FILE* fp = fopen(getCurFile().c_str(), "w");
            if (!fp) {
                error("ե¸Ǥޤ: '%s'\n", getCurFile().c_str());
                goto label;
            }
            fwrite(buf, 1, strlen(buf), fp);
            fclose(fp);

          exist_data = true;
          goto label;
        }
    }
    remove(getCurFile().c_str());   // ƤȤ
    exist_data = false;
label:
    gtk_text_thaw(GTK_TEXT(text));
    g_free(buf);
}

void EditWindow::load()
{
    gtk_text_freeze(GTK_TEXT(text));
    gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
    char* buf = 0;
    FILE* fp = 0;
    gint pos[1] = { 0 };

    struct stat st;
    if (stat(getCurFile().c_str(), &st) == -1)
        goto label;
    fp = fopen(getCurFile().c_str(), "r");
    if (!fp)
        goto label;

    buf = new char[st.st_size + 1];
    fread(buf, 1, st.st_size + 1, fp);
    buf[st.st_size] = 0;
    gtk_editable_insert_text(GTK_EDITABLE(text), buf, strlen(buf), pos);

    delete [] buf;
    fclose(fp);
label:
    gtk_text_thaw(GTK_TEXT(text));
}


void
on_edit_window_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
  EditWindow* this_ = findEditWindow(widget);
  this_->window = widget;
  this_->text = lookup_widget(widget, "text");

  this_->load();

  char buf[100];
  sprintf(buf, "%04d.%02d.%02d", this_->cur_year,
                              this_->cur_month, this_->cur_day);
  gtk_window_set_title(GTK_WINDOW(this_->window), buf);
}

gboolean
on_edit_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    EditWindow* this_ = findEditWindow(widget);
    this_->save();
    calenWindow.updateDay(this_->cur_year, this_->cur_month, this_->cur_day,
                        this_->exist_data);
    editWins.erase(this_);
    itemListWindow.update();
    return FALSE;
}


void
on_text_changed                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
  // TODO: ȥС'*'դ
}

void
on_edit_save_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    EditWindow* this_ = findEditWindow(GTK_WIDGET(button));
    this_->save();
    calenWindow.updateDay(this_->cur_year, this_->cur_month, this_->cur_day,
                        this_->exist_data);
    gtk_widget_destroy(this_->window);
    editWins.erase(this_);
    itemListWindow.update();
}


void
on_edit_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    EditWindow* this_ = findEditWindow(GTK_WIDGET(button));
    gtk_widget_destroy(this_->window);
    editWins.erase(this_);
}

////////////////////////////////////////////////////////////////////////
// CalenWindow

CalenWindow::CalenWindow()
{
}

CalenWindow::~CalenWindow()
{
}

void CalenWindow::updateDay(int y, int m, int d, bool exist_data)
{
    for (int i = 0; i < 2; i++) {
        int year, month, day;
        gtk_calendar_get_date(GTK_CALENDAR(calenWindow.calen[i]),
                (unsigned*) &year, (unsigned*) &month, (unsigned*) &day);
        month++;
        if (year == y && month == m) {
            if (exist_data)
                gtk_calendar_mark_day(GTK_CALENDAR(calenWindow.calen[i]), d);
            else
                gtk_calendar_unmark_day(GTK_CALENDAR(calenWindow.calen[i]), d);
        }
    }
}

gboolean
on_calen_window_delete_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return FALSE;
}


void
on_calen_window_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
  EditWins::const_iterator i;
  for (i = editWins.begin(); i != editWins.end(); i++)
    (*i)->save();

  gtk_main_quit();
}


void
on_calen_window_realize                (GtkWidget       *widget,
                                        gpointer         user_data)
{
  calenWindow.window = widget;
  calenWindow.calen[0] = lookup_widget(widget, "calen1");
  calenWindow.calen[1] = lookup_widget(widget, "calen2");

    time_t tim;
    time(&tim);
    tm* t = localtime(&tim);

    GDate* today = g_date_new_dmy(t->tm_mday,
                        (GDateMonth) (t->tm_mon + 1), t->tm_year + 1900);
    TRACE("today = %04d.%02d.%02d (%d)\n",
                g_date_year(today), g_date_month(today), g_date_day(today),
                g_date_weekday(today));

    gtk_calendar_select_month(GTK_CALENDAR(calenWindow.calen[0]),
                        g_date_month(today) - 1, g_date_year(today));
/*
    g_date_add_months(today, 1);
    gtk_calendar_select_month(GTK_CALENDAR(calenWindow.calen[1]),
                        g_date_month(today) - 1, g_date_year(today));
*/
    gtk_calendar_select_day(GTK_CALENDAR(calenWindow.calen[0]),
                        g_date_day(today));

    g_date_free(today);
}

void openEditWindow(int y, int m, int d)
{
    EditWindow* e = findEditWindow(y, m, d);
    if (!e) {
        e = new EditWindow(y, m, d);
        editWins.insert(e);

        e->window = create_edit_window();
        gtk_widget_show(e->window);
    }
    else {
        Window w = GDK_WINDOW_XWINDOW(GTK_WIDGET(e->text)->window);
        if (w)
            XSetInputFocus(GDK_DISPLAY(), w, RevertToParent, CurrentTime);
    }
    gdk_window_raise(e->window->window);
    gtk_widget_grab_focus(GTK_WIDGET(e->text));
}

void
on_calen_month_changed                 (GtkCalendar     *calendar,
                                        gpointer         user_data)
{
/*
    gtk_signal_emit_stop_by_name(GTK_OBJECT(calendar),
                                     "day_selected");
*/
    guint year, month, day;
    gtk_calendar_get_date(GTK_CALENDAR(calendar), &year, &month, &day);
    month++;
    TRACE("onMonthChanged(): %04d.%02d.%02d\n", year, month, day);

    GDate* date = g_date_new_dmy(day, (GDateMonth) month, year);
    if (calendar == GTK_CALENDAR(calenWindow.calen[0])) {
        g_date_add_months(date, +1);
        gtk_calendar_select_month(GTK_CALENDAR(calenWindow.calen[1]),
                        g_date_month(date) - 1, g_date_year(date));
    }
/*
    else {
        g_date_subtract_months(date, +1);
        gtk_calendar_select_month(GTK_CALENDAR(calenWindow.calen[0]),
                        g_date_month(date) - 1, g_date_year(date));
    }
*/
    g_date_free(date);

    gtk_calendar_freeze(GTK_CALENDAR(calendar));
    gtk_calendar_clear_marks(GTK_CALENDAR(calendar));

    DIR* dir = opendir(getPathName(year, month).c_str());
    if (dir) {
        dirent* ent;
        while ((ent = readdir(dir)) != 0) {
            int f_day;
            if (sscanf(ent->d_name, "%02d", &f_day) == 1) {
                TRACE("mark = '%s'\n", ent->d_name);
                gtk_calendar_mark_day(calendar, f_day);
            }
        }
        closedir(dir);
    }
    gtk_calendar_thaw(GTK_CALENDAR(calendar));
}

static int mouse_button = 0;
    // GtkCalendarϥ˥塼ܥΥåǤday_selectedʥ
    // ȯ롣äơɤΥܥday_selectedƤФ줿Ĵ
    // ɬפ

gboolean
on_calen_button_press_event            (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    mouse_button = event->button;
        // TRUE֤ȡեưʤʤ롣
    return FALSE;
}


gboolean
on_calen_button_release_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    mouse_button = 0;

    if (event->button == MENU_BUTTON) {
        calenWindow.focused = widget;
        GtkWidget* menu = create_r_menu();
        gtk_menu_popup(GTK_MENU(menu), 0, 0, 0, 0, 0, 0);
        // gtk_widget_show(r_menu);
        return TRUE;    // ǥեȽػ
    }
    return FALSE;
}

void
on_calen_day_selected                  (GtkCalendar     *calendar,
                                        gpointer         user_data)
{
    if (mouse_button == MENU_BUTTON) {
        gtk_signal_emit_stop_by_name(GTK_OBJECT(calendar),
                                     "day_selected");
        return;
    }
    else {
        int y, m, d;
        gtk_calendar_get_date(GTK_CALENDAR(calendar),
                        (unsigned*) &y, (unsigned*) &m, (unsigned*) &d);
        m++;
        openEditWindow(y, m, d);
    }
}

////////////////////////////////////////////////////////////////////////


void
on_file_close_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_destroy(calenWindow.window);
}


void
on_help_about_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  MessageBox msg(GTK_WINDOW(calenWindow.window), APP_NAME " " APP_VERSION);
  msg.setVisible(true);
}


void
on_view_edit_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    int y, m, d;
    gtk_calendar_get_date(GTK_CALENDAR(calenWindow.focused),
                        (unsigned*) &y, (unsigned*) &m, (unsigned*) &d);
    m++;
    openEditWindow(y, m, d);
}

void
on_view_list_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* item_list = create_item_list_window();
    gtk_widget_show(item_list);
}

////////////////////////////////////////////////////////////////////////
// ItemListWindow

void ItemListWindow::update()
{
    gtk_clist_freeze(GTK_CLIST(clist));
    gtk_clist_clear(GTK_CLIST(clist));
    gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
    gtk_clist_set_auto_sort(GTK_CLIST(clist), TRUE);

    char title[100];
    title[0] = '\0';
    Date month = start;
    for (int i = 0; i < 2; i++, month = month.addMonths(1)) {
        sprintf(title + strlen(title), "%04d.%02d",
                        month.getYear(), month.getMonth());
        if (!i)
            strcat(title, " - ");

        string path = getPathName(month.getYear(), month.getMonth());
        DIR* dir = opendir(path.c_str());
        if (!dir)
            continue;

        dirent* ent;
        while ((ent = readdir(dir)) != 0) {
            int f_day;
            if (sscanf(ent->d_name, "%02d", &f_day) < 1)
                continue;
            FILE* fp = fopen((path + "/" + ent->d_name).c_str(), "r");
            if (!fp)
                continue;

            int ch;
            int sp_cnt = 0;
            string text;
            while ((ch = fgetc(fp)) != EOF && text.length() < 50) {
                if (isspace(ch)) {
                    sp_cnt++;
                    continue;
                }
                if (sp_cnt) {
                    if (sp_cnt > 3)
                        text += "...";
                    else
                        text += " ";
                    sp_cnt = 0;
                }
                text += (char) ch;
                if (ch >= 0x80) {
                    ch = fgetc(fp);
                    text += (char) ch;
                }
            }
            fclose(fp);

            const char* p[2];
            char buf[100];
            Date l_day(month.getYear(), month.getMonth(), f_day);
            sprintf(buf, "%04d.%02d.%02d",
                            l_day.getYear(), l_day.getMonth(), l_day.getDay());
            p[0] = buf;
            p[1] = text.c_str();
            int row = gtk_clist_append(GTK_CLIST(clist), const_cast<char**>(p));
            gtk_clist_set_row_data(GTK_CLIST(clist),
                row, (void*) l_day.getJulian());
        }
        closedir(dir);
    }
    gtk_window_set_title(GTK_WINDOW(window), title);

    gtk_clist_thaw(GTK_CLIST(clist));
}

void
on_item_list_window_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
    itemListWindow.window = widget;
    itemListWindow.clist = lookup_widget(widget, "clist");

    int y, m, d;
    gtk_calendar_get_date(GTK_CALENDAR(calenWindow.calen[0]),
                        (unsigned*) &y, (unsigned*) &m, (unsigned*) &d);
    m++;
    itemListWindow.start = Date(y, m, d);
    itemListWindow.update();
}


void
on_clist_select_row                    (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    int j = (int) gtk_clist_get_row_data(clist, row);
    Date date(j);
    openEditWindow(date.getYear(), date.getMonth(), date.getDay());
}



gboolean
on_clist_button_release_event          (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}

