// Q's C++ Library
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

#include <stdio.h>
#include <sys/stat.h>   // mkdir
#include <X11/Xlib.h>
#include "misc.h"

//////////////////////////////////////////////////////////////////////
// X11

int getColor(Display* disp, const char* name)
{
    assert(disp != 0 && name != 0);
    Colormap cmap;
    XColor c0, c1;
    cmap = DefaultColormap(disp, 0);
    XAllocNamedColor(disp, cmap, name, &c1, &c0);
    return c1.pixel;
}

XFontSet createFont(Display* disp, const char* name)
{
    XFontSet fs = 0;
    char** miss;
    int miss_count;
    char* def_string;

    fs = XCreateFontSet(disp, name, &miss, &miss_count, &def_string);
    if (!fs) {
        TRACE("cannot create fontset.\n");
        return 0;
    }

    if (miss_count > 0) {
        TRACE("warning: cannot load: ");
        for (int i = 0; i < miss_count; i++)
            TRACE("%s ", miss[i]);
        TRACE("\n");
        TRACE("fs locale = '%s'\n", XLocaleOfFontSet(fs));
        XFreeStringList(miss);
    }
    return fs;
}

bool isToplevelWindow(Display* disp, Window w)
{
    Window root, parent, *children;
    unsigned int count;
    XQueryTree(disp, w, &root, &parent, &children, &count);
    XFree(children);
    return root == parent;
}

//////////////////////////////////////////////////////////////////////

void error(const char* format, ...)
{
    va_list args;
    va_start(args, format);
    // g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, format, args);
            // g_log()˲Ԥ
    vfprintf(stderr, format, args);
    va_end(args);
}

int char_length(const char* b, const char* e)
    // [b, e)ʸ֤
{
    int count = 0;
    while (*b && b < e) {
        b += mblen(b, MB_CUR_MAX);
        count++;
    }
    return count;
}

int byte_length(const char* s, int clen)
{
    const char* p = s;
    while (*p && --clen >= 0)
        p += mblen(p, MB_CUR_MAX);
    return p - s;
}

bool createFolder(const char* path)
    // ǽ
    //      եʳؤƤ˺롣
    // 
    //      path'\'դƤդʤƤɤ
    // 
    //      TRUE: ե¸ߤ롣
    //      FALSE: 
{
    assert(path);

    string cur;
    const char* p = path;
    if (path[0] == '/' || path[1] == '\\')
        p++;

    while (*p) {
        cur += '/';
        while (*p && *p != '/' && *p != '\\') {
#ifdef WIN32
            if (::IsDBCSLeadByte(*p))
                cur += *p++;
#endif
            cur += *p++;
        }

        struct stat st;
        if (stat(cur.c_str(), &st) < 0) {
            if (errno == ENOENT) {
                if (mkdir(cur.c_str(), 0755) != 0 && errno != EEXIST) {
                    perror("mkdir");
                    return false;
                }
            }
            else {
                perror("stat");
                return false;
            }
        }

        if (*p)
            p++;
    }
    return true;
}

//////////////////////////////////////////////////////////////////////
// CPoint

CPoint::CPoint()
{
    x = 0; y = 0;
}

CPoint::CPoint(int x_, int y_)
{
    x = x_; y = y_;
}

CPoint::CPoint(const XPoint& xp)
{
    x = xp.x; y = xp.y;
}

CPoint::~CPoint()
{
}

CPoint CPoint::operator + (const XPoint& a) const
{
    return CPoint(x + a.x, y + a.y);
}

CPoint CPoint::operator - (const XPoint& a) const
{
    return CPoint(x - a.x, y - a.y);
}

CPoint& CPoint::operator = (const XPoint& a)
{
    x = a.x; y = a.y;
    return *this;
}

//////////////////////////////////////////////////////////////////////
// CRect

CRect::CRect()
{
    x = y = width = height = 0;
}

CRect::CRect(int x_, int y_, int w, int h)
{
    x = x_; y = y_; width = w; height = h;
}

CRect::CRect(const XRectangle& a)
{
    x = a.x; y = a.y; width = a.width; height = a.height;
}

CRect::~CRect()
{
}

CRect& CRect::operator = (const XRectangle& a)
{
    x = a.x; y = a.y; width = a.width; height = a.height;
    return *this;
}

