// Q's C++ Library
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

// ١ؿ

#ifndef QSNICOLA_MISC
#define QSNICOLA_MISC

#include <vector>
#include <list>
#include <set>
#include <string>
#include <X11/Xlib.h>

#include "debug.h"
using namespace std;

////////////////////////////////////////////////////////////////////////

inline char* skip_space(const char* p)
{
    while (*p && isspace(*p))
        p++;
    return const_cast<char*>(p);
}

inline unsigned char* skip_space(const unsigned char* p)
{
    while (*p && isspace(*p))
        p++;
    return const_cast<unsigned char*>(p);
}

extern bool createFolder(const char* path);
extern int char_length(const char* b, const char* e);
extern int byte_length(const char* s, int char_len);
extern void error(const char* format, ...);

/*
//  <algorithm>ȤФ褤
template <class _T>
const _T& my_min(const _T& x, const _T& y)
{
    return x < y ? x : y;
}

template <class _T>
const _T& my_max(const _T& x, const _T& y)
{
    return x > y ? x : y;
}
*/

////////////////////////////////////////////////////////////////////////
// ptr_vector

template <class _T>
class ptr_vector: public vector<_T>
{
    typedef vector<_T> super;
public:
    ptr_vector() { }
    virtual ~ptr_vector() { clear(); }
    void clear() {
        iterator i;
        for (i = begin(); i != end(); i++)
            delete *i;
        super::clear();
    }
    iterator erase(iterator p) {
        delete *p;
        return super::erase(p);
    }
};

////////////////////////////////////////////////////////////////////////
// ptr_list

template <class _T>
class ptr_list: public list<_T>
{
public:
    ptr_list() { }
    virtual ~ptr_list() { clear(); }
    void clear() {
        iterator i;
        for (i = begin(); i != end(); i++)
            delete *i;
        list<_T>::clear();
    }
};

////////////////////////////////////////////////////////////////////////
// ptr_set

template <class _T, class _COMP = less<_T> >
class ptr_set: public set<_T, _COMP>
{
    typedef set<_T, _COMP> super;
public:
    ptr_set() { }
    virtual ~ptr_set() { clear(); }

    void clear() {
        iterator i;
        for (i = begin(); i != end(); i++)
            delete *i;
        super::clear();
    }

    size_type erase(const key_type& x) {
        _T ptr = x;
        size_t retval = super::erase(x);
        delete ptr;
        return retval;
    }

    void erase(iterator i) {
        _T ptr = *i;
        super::erase(i);
        delete ptr;
    }
};

////////////////////////////////////////////////////////////////////////
// SizeList

template <class _T>
class SizeList: public vector<_T>
{
public:
    _T sum(int first, int last) const
        // [first, last)ι
    {
        _T r = 0;
        const_iterator i;
        for (i = begin() + first; i != end() && i != begin() + last; i++)
            r += *i;
        return r;
    }
    void replace(int pos, int n, _T val)
        // posnǤĤval֤
    {
        erase(begin() + pos, begin() + pos + n - 1);
        operator [](pos) = val;
    }
};

////////////////////////////////////////////////////////////////////////
// X11

extern int getColor(Display* disp, const char* name);
extern XFontSet createFont(Display* disp, const char* name);
extern bool isToplevelWindow(Display* disp, Window w);

////////////////////////////////////////////////////////////////////////
// CPoint

class CPoint: public XPoint
{
public:
    CPoint();
    CPoint(int x, int y);
    CPoint(const XPoint& );
    virtual ~CPoint();

    CPoint operator + (const XPoint& a) const;
    CPoint operator - (const XPoint& a) const;
    CPoint& operator = (const XPoint& a);
};

////////////////////////////////////////////////////////////////////////
// CRect

class CRect: public XRectangle
{
public:
    CRect();
    CRect(int x, int y, int width, int height);
    CRect(const XRectangle& );
    virtual ~CRect();

    CRect& operator = (const XRectangle& a);
};

////////////////////////////////////////////////////////////////////////
// CDimension

class CDimension
{
public:
    int width, height;
    CDimension(): width(0), height(0) { }
};

#endif  // QSNICOLA_MISC
