// Q's C++ Library
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

#ifndef QSLIB_OBSERVER
#define QSLIB_OBSERVER

#include <set>
using namespace std;

//////////////////////////////////////////////////////////////////////
// Observer

class Observer
    // interface java.util.Observer
{
    friend class Observable;
    virtual void update(Observable* o, void* arg) = 0;
};

//////////////////////////////////////////////////////////////////////
// Observable

class Observable
    // java.util.Observable
{
    typedef set<Observer*> Observers;
    Observers _os;
    bool changed;

public:
    virtual ~Observable()
    {
    }

    int countObservers() const
    {
        return _os.size();
    }

    void attachObserver(Observer* o)
    {
        assert(o);
        _os.insert(o);
    }

    void detachObserver(Observer* o)
    {
        Observers::const_iterator i = _os.find(o);
        if (i != _os.end())
            _os.erase(i);
    }

    bool isChanged() const
    {
        return changed;
    }

    void setChanged(bool c)
    {
        changed = c;
    }

protected:
    Observable(): changed(false)
    {
    }

    virtual void notifyObservers(void* arg = 0)
    {
        if (isChanged()) {
            Observers::iterator i;
            for (i = _os.begin(); i != _os.end(); i++)
                (*i)->update(this, arg);
            setChanged(false);
        }
    }
};

#endif
