// Q's C++ Library
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

#include <misc.h>
#include <debug.h>
#include "xim.h"

////////////////////////////////////////////////////////////////////////
// InputMethod

InputMethod::InputMethod(): disp(NULL), im(0)
{
}

InputMethod::~InputMethod()
{
    close();
}

void InputMethod::onImInstantiate(Display* disp, XPointer client_data, XPointer call_data)
{
    TRACE("%s\n", __func__);
    InputMethod* this_ = (InputMethod*) client_data;

    XUnregisterIMInstantiateCallback(disp, NULL, NULL, NULL,
                            onImInstantiate, (XPointer) this_);
        // Ƥʤȡ¿Ť˸ƤФ

    Listeners::const_iterator i;
    for (i = this_->listeners.begin(); i != this_->listeners.end(); i++)
        (*i)->inputMethodInstantiated();
}

void InputMethod::onImServerDestroy(XIM im_, InputMethod* this_, XPointer /*call_data*/)
{
    TRACE("%s\n", __func__);

    if (this_->im == im_) {
        this_->im = 0;

        Listeners::const_iterator i;
        for (i = this_->listeners.begin(); i != this_->listeners.end(); i++)
            (*i)->inputMethodDestroyed();

        XRegisterIMInstantiateCallback(this_->disp, NULL, NULL, NULL,
                            onImInstantiate, (XPointer) this_);
    }
}

void InputMethod::close()
{
    if (im) {
        XCloseIM(im);
        im = 0;
    }
}

XIMStyle InputMethod::selectInputStyle(const XIMStyle* styles) const
    // IMС˥ݡȤϥ䤤碌޾פ롣
{
    if (!im)
        return 0;

    XIMStyles* im_styles = NULL;
    const char* mes = XGetIMValues(im, XNQueryInputStyle, &im_styles, NULL);
    if (mes) {
        TRACE("XGetIMValues() failed: '%s'\n", mes);
        return 0;
    }
#if DEBUG > 1
    for (int iii = 0; iii < im_styles->count_styles; iii++)
        printStyle(im_styles->supported_styles[iii]);
#endif
    const XIMStyle* p;
    for (p = styles; *p; p++) {
        for (int i = 0; i < im_styles->count_styles; i++) {
            if (*p == im_styles->supported_styles[i]) {
                XFree(im_styles);
                return *p;
            }
        }
    }
    XFree(im_styles);
    return 0;
}

extern "C" {
    // XFree86 bug: X11/Xlib.h˺Ƥ롣
    char* XSetIMValues(XIM im, ...);
}

bool InputMethod::open(Display* d)
    // IM򳫤 = IMС³
{
    assert(d);
    disp = d;
    im = XOpenIM(disp, NULL, NULL, NULL);
        // XmbLookupString()δؿ֤ʸϤλΥ
        // ʤΥ󥳡ǥ󥰡ˤǥ󥳡ɤ롣
    if (!im) {
        error("warning: cannot open IM\n");
        // IMСͭˤʤäǸƤӽФƤ餦
        XRegisterIMInstantiateCallback(disp, NULL, NULL, NULL,
                                    onImInstantiate, (XPointer) this);
        return false;
    }
    TRACE("im locale = '%s'\n", XLocaleOfIM(im));

#if 0
    changeLocale("C");
        // XOpenIM()ѹƤXmbLookupString()Υ󥳡ɤѤʤ
#endif

    // IMСȤ˸ƤӽФƤ餦
    XIMCallback callback = { (XPointer) this, (XIMProc) onImServerDestroy };
    const char* mes = XSetIMValues(im, XNDestroyCallback, &callback, NULL);
    if (mes)
        error("XSetIMValues failed: '%s'\n", mes);

    return true;
}

bool InputMethod::is_open() const
{
    return im != 0;
}

XIM InputMethod::getXIM() const
{
    return im;
}

void InputMethod::attachInputMethodListener(InputMethodListener* l)
{
    listeners.push_back(l);
}
