// mini-notepad
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

#include "../config.h"

#include <cstdio>
#include <set>
#include <time.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include <misc.h>
#include <dialog.h>
#include <document.h>
#include <misc-gtk.h>

using namespace std;

////////////////////////////////////////////////////////////////////////

#define APP_NAME    "mini-notepad"
#define APP_VERSION VERSION

////////////////////////////////////////////////////////////////////////
// NotepadDoc

NotepadDoc::NotepadDoc(NotepadWindow* f): frame(f)
{
}

NotepadDoc::~NotepadDoc()
{
}

bool NotepadDoc::save()
{
    FILE* fp = fopen(filename.c_str(), "w");
    if (!fp) {
        error("ե뤬ʤ: '%s'\n", filename.c_str());
        return false;
    }

    gtk_text_freeze(GTK_TEXT(frame->text));
    char* const buf = gtk_editable_get_chars(GTK_EDITABLE(frame->text), 0, -1);
    gtk_text_thaw(GTK_TEXT(frame->text));
    fwrite(buf, 1, strlen(buf), fp);
    g_free(buf);

    if (fclose(fp)) {
        error("ե¸˼ԡ'%s'\n", filename.c_str());
        return false;
    }

    setModified(false);
    updateTitle();
    return true;
}

////////////////////////////////////////////////////////////////////////
// Frames

Frames frames;

NotepadWindow* findNotepadWindow(GtkWidget* widget)
{
    widget = getWindow(widget);
    Frames::const_iterator i;
    for (i = frames.begin(); i != frames.end(); i++) {
        if ((*i)->window == widget)
            return *i;
        else if ((*i)->searchDlg && (*i)->searchDlg->window == widget)
            return *i;
        else if ((*i)->replaceDlg && (*i)->replaceDlg->window == widget)
            return *i;
    }
    return NULL;
}

////////////////////////////////////////////////////////////////////////
// NotepadWindow

NotepadWindow::NotepadWindow(): window(NULL), text(NULL),
                            searchDlg(NULL), replaceDlg(NULL)
{
    doc = new NotepadDoc(this);
}

NotepadWindow::~NotepadWindow()
{
    delete doc;
    delete searchDlg;
    delete replaceDlg;
}

void NotepadWindow::updateTitle(const string& title)
{
    gtk_window_set_title(GTK_WINDOW(window), title.c_str());
}

void NotepadWindow::onSearchMenuMap(GtkWidget* widget, NotepadWindow* this_)
{
    GtkWidget* search = lookup_widget(this_->window, "search_search");
    GtkWidget* replace = lookup_widget(this_->window, "search_replace");

    GTK_CHECK_MENU_ITEM(search)->active
        = this_->searchDlg
                && GTK_WIDGET_VISIBLE(this_->searchDlg->window) != 0;
    GTK_CHECK_MENU_ITEM(replace)->active
        = this_->replaceDlg
                && GTK_WIDGET_VISIBLE(this_->replaceDlg->window) != 0;
}

void NotepadWindow::onWindowWindow(GtkMenuItem* menuitem, NotepadWindow* target)
{
    XSetInputFocus(GDK_DISPLAY(),
            GDK_WINDOW_XWINDOW(GTK_WIDGET(target->text)->window),
            RevertToParent, CurrentTime);
         // gtk+ 1.2.3: ɤX11áʤȥեʤ
    gdk_window_raise(target->window->window);
    gtk_widget_grab_focus(GTK_WIDGET(target->text));
}

void NotepadWindow::onWindowMenuMap(GtkWidget* widget, NotepadWindow* this_)
{
    GtkWidget* menu = lookup_widget(widget, "window_menu_menu");

    gtk_container_foreach(GTK_CONTAINER(menu),
                                (GtkCallback) gtk_widget_destroy, NULL);

    Frames::const_iterator i;
    for (i = frames.begin(); i != frames.end(); i++) {
        GtkWidget* item = gtk_menu_item_new_with_label(
                                    (*i)->doc->getTitle(*i).c_str());
        gtk_menu_append(GTK_MENU(menu), item);
        gtk_widget_show(item);
        gtk_signal_connect(GTK_OBJECT(item), "activate",
                                GTK_SIGNAL_FUNC(onWindowWindow), *i);
    }
}

void
on_notepad_window_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(widget);
    assert(frame);
    frame->text = lookup_widget(widget, "text");
    frame->h_tree = lookup_widget(widget, "h_tree");
    assert(frame->h_tree);

    GtkWidget* search_menu = lookup_widget(widget, "search_menu_menu");
    assert(search_menu);
    gtk_signal_connect(GTK_OBJECT(search_menu), "map",
            GTK_SIGNAL_FUNC(NotepadWindow::onSearchMenuMap), frame);

    GtkWidget* window_menu = lookup_widget(widget, "window_menu_menu");
    assert(window_menu);
    gtk_signal_connect(GTK_OBJECT(window_menu), "map",
            GTK_SIGNAL_FUNC(NotepadWindow::onWindowMenuMap), frame);

    GtkWidget* vscrollbar = lookup_widget(widget, "text_vscrollbar");
    GtkAdjustment* adj = gtk_range_get_adjustment(GTK_RANGE(vscrollbar));
    gtk_text_set_adjustments(GTK_TEXT(frame->text), NULL, adj);
}

gboolean
on_notepad_window_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(widget);
    assert(frame);
    if (frame->close())
        gtk_widget_destroy(frame->window);

    return TRUE;
}


void
on_notepad_window_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(object));
    assert(frame);

    if (frames.size())
        frames.erase(frame);

    if (!frames.size())
        gtk_main_quit();
}


void
on_text_changed                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(editable));
    assert(frame);
    if (!frame->doc->isModified()) {
        frame->doc->setModified(true);
        frame->doc->updateTitle();
    }
    // frame->updateHeadingTree();
}

const char* h_start = "\n";

void NotepadWindow::updateHeadingTree()
{
    // TRACE("updateHeadingTree()\n");

    gtk_clist_freeze(GTK_CLIST(h_tree));
    gtk_clist_clear(GTK_CLIST(h_tree));

    const char* t[1];
    Headings::const_iterator i;
    for (i = headings.begin(); i != headings.end(); i++) {
        t[0] = (string("\'") + (*i)->name + "\'").c_str();
        int row = gtk_clist_append(GTK_CLIST(h_tree), const_cast<char**>(t));
        gtk_clist_set_row_data(GTK_CLIST(h_tree), row, *i);
    }

/*
    char* const e = gtk_editable_get_chars(GTK_EDITABLE(text), 0, -1);
    headings.clear();

    const char* p = e;
    while ((p = strstr(p, h_start)) != NULL) {
        p += strlen(h_start);
        Heading h;
        h.pos = char_length(e, p);
        while (*p && *p != '\n')
            h.name += *p++;
        headings.push_back(h);

        const char* t[1];
        t[0] = (string("-") + h.name).c_str();
        gtk_clist_append(GTK_CLIST(h_tree), const_cast<char**>(t));
    }
*/
    gtk_clist_thaw(GTK_CLIST(h_tree));
}

Headings::iterator NotepadWindow::insertHeading(Heading* h)
{
    Headings::iterator i;
    for (i = headings.begin(); i != headings.end(); i++) {
        if (h->pos <= (*i)->pos)
            break;
    }
    return headings.insert(i, h);
}

void
on_text_insert_text                    (GtkEditable     *editable,
                                        gchar           *new_text,
        gint    new_text_length,    // Хñ
        gint*   position,           // ʸñ
                                        gpointer         user_data)
{
#if DEBUG >= 2
    TRACE("on_text_insert_text(): *pos = %d, len = %d\n",
                                        *position, new_text_length);
    for (int ii = 0; ii < new_text_length; ii++)
        TRACE("%02x ", (unsigned char) new_text[ii]);
    TRACE("\n");
#endif

    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(editable));
    assert(frame);
/*
    bool to_update = false;
    if (strstr(new_text, h_start) != NULL)
        to_update = true;
    else if (*position && !strncmp(new_text, "", strlen(""))) {
        TRACE("1\n");
        char* const e = gtk_editable_get_chars(editable,
                                *position - 1, *position);
        if (e && !strncmp(e, "\n", strlen("\n")))
            to_update = true;
        g_free(e);
    }
    else if (!strncmp(new_text + new_text_length - strlen("\n"),
                                            "\n", strlen("\n"))) {
        TRACE("2\n");
        char* const e = gtk_editable_get_chars(editable,
                *position, *position + mbstowcs(NULL, "", strlen("")));
        if (e && !strncmp(e, "", strlen("")))
            to_update = true;
        g_free(e);
    }
*/

    int h_start_len = mbstowcs(NULL, h_start, strlen(h_start));

    Headings::reverse_iterator rev_i;
    for (rev_i = frame->headings.rbegin(); rev_i != frame->headings.rend(); rev_i++) {
        if ((*rev_i)->pos - h_start_len >= *position)
            (*rev_i)->pos += char_length(new_text, new_text + new_text_length);
        else
            break;
    }

    int h_len = 0;
    Headings::iterator cur;
    for (cur = frame->headings.begin(); cur != frame->headings.end(); cur++) {
            // ܤƤ븫Ф
        h_len = mbstowcs(NULL, (*cur)->name.c_str(), (*cur)->name.length());
        if ((*cur)->pos + h_len >= *position)
            break;
    }

#if DEBUG >= 1
    if (cur != frame->headings.end()) {
        TRACE("cur.name = '%s', pos = %d\n", (*cur)->name.c_str(), (*cur)->pos);
    }
#endif

    bool to_update = false;
    const char* t = new_text;
    const char* end_p = new_text + new_text_length;
    if (cur != frame->headings.end()
                && *position > (*cur)->pos - h_start_len
                && *position < (*cur)->pos) {
        // Ф''ľ
        TRACE("5\n");
        to_update = true;
        cur = frame->headings.erase(cur);
    }
    else if (cur != frame->headings.end()
                && *position >= (*cur)->pos
                && *position <= (*cur)->pos + h_len) {
        // Ǥˤ븫Ф
        int b_pos = byte_length((*cur)->name.c_str(), *position - (*cur)->pos);
        // TRACE("b_pos = %d, *position = %d\n", b_pos, *position);
        while (*t && *t != 0x0d && *t != 0x0a && t < end_p) {
            // TRACE("*t = %02x\n", (unsigned char) *t);
            to_update = true;
            (*cur)->name.insert(b_pos, t, 1);
            b_pos++;
            t++;
        }
        if (t < end_p && *t == 0x0d || *t == 0x0a) {
            to_update = true;
            (*cur)->name.erase(b_pos);
                // Ԥ줿ȤθФûʤ뤳Ȥ롣
                // ǺФϡΥɤǼθФȤѤ뤳Ȥ롣
        }
    }

    // Фʤ뤫
    const char* p;
    Heading* h = NULL;
    char* /*const*/ e;
    for (p = t; p < end_p; p += mblen(p, MB_CUR_MAX)) {
        if (*p == '\n' && p + 1 == end_p) {
            // TRACE("3\n");
            e = gtk_editable_get_chars(editable, *position,
                    *position + mbstowcs(NULL, "", strlen("")));
            if (e && !strncmp(e, "", strlen(""))) {
                g_free(e);
                h = new Heading();
                h->name = "";
                h->pos = *position + char_length(new_text, p) + h_start_len;
                e = gtk_editable_get_chars(editable,
                        *position + mbstowcs(NULL, "", strlen("")),
                         -1);
                // TRACE("4\n");
                goto l2;
            }
            g_free(e);
            continue;
        }
        else if (p == new_text && !strncmp(p, "", strlen(""))) {
            e = gtk_editable_get_chars(editable, *position - strlen("\n"), *position);
            if (e && !strncmp(e, "\n", strlen("\n"))) {
                g_free(e);
                h = new Heading();
                h->pos = *position + mbstowcs(NULL, "", strlen(""));
                p += strlen("");
                goto l1;
            }
            g_free(e);
            continue;
        }
        else if (strncmp(p, h_start, strlen(h_start)))
            continue;

        h = new Heading();
        p += strlen(h_start);
        h->pos = *position + char_length(new_text, p);
    l1:
        h->name = "";
        while (*p && *p != 0x0d && *p != 0x0a && p < end_p)
            h->name += *p++;
        if (!*p || p == end_p) {
            // ''ȤΥƥȤ⸫Фˤʤ
            e = gtk_editable_get_chars(editable, *position, -1);
        l2:
            const char* ee = e;
            while (*ee && *ee != 0x0d && *ee != 0x0a)
                h->name += *ee++;
            g_free(e);
        }
        else
            --p;
        to_update = true;
        cur = frame->insertHeading(h) + 1;
    }

    if (to_update)
        frame->updateHeadingTree();
}

void
on_text_delete_text                    (GtkEditable     *editable,
                                        gint             start_pos,
                                        gint             end_pos,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(editable));
    assert(frame);

    int h_start_len = mbstowcs(NULL, h_start, strlen(h_start));

    bool to_update = false;
    int h_len = 0;
    Headings::iterator cur;
    for (cur = frame->headings.begin(); cur != frame->headings.end(); ) {
        h_len = mbstowcs(NULL, (*cur)->name.c_str(), (*cur)->name.length());
        if ((*cur)->pos + h_len <= start_pos)
            /* empty */ ;
        else if ((*cur)->pos + h_len > start_pos && (*cur)->pos <= start_pos) {
            // Ƥޤϰ
            to_update = true;
            int b_pos = byte_length((*cur)->name.c_str(), start_pos - (*cur)->pos);
            int e_pos = byte_length((*cur)->name.c_str(), end_pos - (*cur)->pos);
            (*cur)->name.erase(b_pos, e_pos - b_pos);
        }
        else if ((*cur)->pos > start_pos && (*cur)->pos - h_start_len <= start_pos) {
            // ''κ -> Ф
            to_update = true;
            cur = frame->headings.erase(cur);
            continue;
        }
        else if ((*cur)->pos - h_start_len < end_pos) {
            // ''κ -> Ф
            to_update = true;
            cur = frame->headings.erase(cur);
            continue;
        }
        else {
            // ־ι
            (*cur)->pos -= end_pos - start_pos;
        }
        cur++;
    }

    // ˤäƿ˸Ф뤳Ȥ
    char* const e1 = gtk_editable_get_chars(editable, start_pos - 1, start_pos);
    char* const e2 = gtk_editable_get_chars(editable,
                end_pos, end_pos + mbstowcs(NULL, "", strlen("")));
    if (e1 && e2 && !strncmp(e1, "\n", strlen("\n"))
                        && !strncmp(e2, "", strlen(""))) {
        Heading* h = new Heading();
        h->pos = start_pos + mbstowcs(NULL, "", strlen(""));
        char* const e = gtk_editable_get_chars(editable,
                    end_pos + mbstowcs(NULL, "", strlen("")), -1);
        TRACE("ep = '%s'\n", e);
        if (e) {
            const char* ee = e;
            while (*ee && *ee != '\n')
                h->name += *ee++;
            g_free(e);
        }
        to_update = true;
        frame->insertHeading(h);
    }
    g_free(e1); g_free(e2);

    if (to_update)
        frame->updateHeadingTree();
/*
    char* const e = gtk_editable_get_chars(editable, start_pos, end_pos);
    if (e) {
        if (strstr(e, h_start) != NULL)
            to_update = true;
        else if (start_pos && !strncmp(e, "", strlen(""))) {
            char* const ee = gtk_editable_get_chars(editable, start_pos - 1, start_pos);
            if (ee && !strncmp(ee, "\n", strlen("\n")))
                to_update = true;
            g_free(ee);
        }
        else if (!strncmp(e + byte_length(e, end_pos - start_pos) - 1,
                        "\n", strlen("\n"))) {
            char* const ee = gtk_editable_get_chars(editable, end_pos,
                end_pos + mbstowcs(NULL, "", strlen("")));
            if (ee && !strncmp(e, "", strlen("")))
                to_update = true;
            g_free(ee);
        }
        g_free(e);
    }
*/
}

void
on_h_tree_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(clist));
    assert(frame);

    XSetInputFocus(GDK_DISPLAY(),
            GDK_WINDOW_XWINDOW(GTK_WIDGET(frame->text)->window),
            RevertToParent, CurrentTime);
    gtk_widget_grab_focus(GTK_WIDGET(frame->text));

    Heading* h = (Heading*) gtk_clist_get_row_data(clist, row);
    TRACE("pos = %d\n", h->pos);
    gtk_text_set_point(GTK_TEXT(frame->text), h->pos);
}

////////////////////////////////////////////////////////////////////////

void
on_file_new_window_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* window = new NotepadWindow();
    window->window = create_notepad_window();
    window->doc->attachController(window);
    window->doc->updateTitle();
    frames.insert(window);
    gtk_widget_show(window->window);
}


void
on_file_new_reuse_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);

    if (!frame->close())
        return;

    gtk_text_freeze(GTK_TEXT(frame->text));
    gtk_editable_delete_text(GTK_EDITABLE(frame->text), 0, -1);
    gtk_text_thaw(GTK_TEXT(frame->text));

    frame->headings.clear();
    frame->updateHeadingTree();

    frame->doc->setModified(false);
    frame->doc->updateTitle();
}

bool loadFile(NotepadWindow* frame, const string& filename)
{
    FILE* fp = fopen(filename.c_str(), "r");
    if (!fp) {
        error("ե뤬Ĥޤ: '%s'\n", filename.c_str());
        return false;
    }

    char buf[1000];
    gtk_text_freeze(GTK_TEXT(frame->text));
    gtk_editable_delete_text(GTK_EDITABLE(frame->text), 0, -1);
    while (fgets(buf, sizeof(buf), fp) != 0) {
        gtk_text_insert(GTK_TEXT(frame->text), 0,
                &GTK_WIDGET(frame->text)->style->black, 0,
                buf, strlen(buf));
    }
    fclose(fp);

    frame->headings.clear();
    char* const e = gtk_editable_get_chars(GTK_EDITABLE(frame->text), 0, -1);
    const char* p = e;
    while ((p = strstr(p, h_start)) != NULL) {
        p += strlen(h_start);
        Heading* h = new Heading();
        h->pos = char_length(e, p);
        while (*p && *p != '\n')
            h->name += *p++;
        frame->headings.push_back(h);

        if (*p == '\n')
            --p;
    }
    frame->updateHeadingTree();
    g_free(e);

    gtk_text_thaw(GTK_TEXT(frame->text));

    frame->doc->filename = filename;
    frame->doc->setModified(false);
    frame->doc->updateTitle();
    return true;
}

void
on_file_open_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);

    if (!frame->close())
        return;

    FileDialog dlg(GTK_WINDOW(frame->window), "ե򳫤");
    if (dlg.setVisible(true) == FileDialog::IDOK)
        loadFile(frame, dlg.getFile());
}


void
on_file_save_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    frame->save();
}


void
on_file_save_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    frame->saveAs();
}


void
on_file_close_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    if (frame->close())
        gtk_widget_destroy(frame->window);
}


void
on_edit_cut_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    gtk_editable_cut_clipboard(GTK_EDITABLE(frame->text));
}


void
on_edit_copy_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    gtk_editable_copy_clipboard(GTK_EDITABLE(frame->text));
}


void
on_edit_paste_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    gtk_editable_paste_clipboard(GTK_EDITABLE(frame->text));
}


void
on_edit_clear_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    GtkEditable* text = GTK_EDITABLE(frame->text);
    gtk_editable_delete_text(text, text->selection_start_pos,
                                            text->selection_end_pos);
}


void
on_edit_select_all_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    gtk_editable_select_region(GTK_EDITABLE(frame->text), 0, -1);
}


void
on_insert_date_time_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    static const char* week[] = {
                        "", "", "", "", "", "", "" };
    time_t gmt;
    time(&gmt);
    tm* local = localtime(&gmt);
    char buf[100];
    sprintf(buf, "%04d.%02d.%02d(%s) %02d:%02d",
                local->tm_year + 1900, local->tm_mon + 1, local->tm_mday,
                week[local->tm_wday], local->tm_hour, local->tm_min);
    gint pos[1];
    pos[0] = gtk_editable_get_position(GTK_EDITABLE(frame->text));
    gtk_editable_insert_text(GTK_EDITABLE(frame->text), buf,
                                                    strlen(buf), pos);
}


void
on_search_search_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);

    if (!frame->searchDlg)
        frame->searchDlg = new SearchDlg();
    gtk_widget_show(frame->searchDlg->window);
}


void
on_search_replace_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);

    if (!frame->replaceDlg)
        frame->replaceDlg = new ReplaceDlg();
    gtk_widget_show(frame->replaceDlg->window);
}


void
on_help_about_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(menuitem));
    assert(frame);
    MessageBox msg(GTK_WINDOW(frame->window), APP_NAME " " APP_VERSION);
    msg.setVisible(true);
}

////////////////////////////////////////////////////////////////////////
// SearchDlg

SearchDlg::SearchDlg()
{
    window = create_search_window();
}

SearchDlg::~SearchDlg()
{
}

void
on_search_cancel                       (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget* window = lookup_widget(GTK_WIDGET(button), "search_window");
    gtk_widget_hide(window);
}


gboolean
on_search_window_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    gtk_widget_hide(widget);
    return TRUE;
}


void
on_search_window_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(widget);
    SearchDlg* this_ = frame->searchDlg;

    this_->search_entry = lookup_widget(widget, "search_entry");
}

void searchForward(GtkWidget* text, const char* find_str, const char* replace)
{
    int pos;
    GtkEditable* editable = GTK_EDITABLE(text);
    if (editable->selection_start_pos != editable->selection_end_pos)
        pos = editable->selection_end_pos;
    else
        pos = gtk_editable_get_position(editable);
    TRACE("pos = %d\n", pos);

    char* const str = gtk_editable_get_chars(editable, pos, -1);
    if (!str || str[0] == 0) {
        g_free(str);
        return;
    }

    const char* p = str;
    for (int i = mbstowcs(NULL, str, strlen(str)); i > 0; i--) {
        if (!strncmp(p, find_str, strlen(find_str))) {
            int len = mbstowcs(NULL, find_str, strlen(find_str));
            if (!replace) {
                gtk_editable_select_region(editable, pos, pos + len);
                    // posʸñ
                // gtk_editable_set_position(GTK_EDITABLE(frame->text), pos + len);
                    // gtk_editable_set_position()Ƥޤ
            }
            else {
                gtk_text_freeze(GTK_TEXT(text));
                gtk_editable_delete_text(editable, pos, pos + len);
                if (replace[0] != '\0') {
                    int pos_d = pos;
                    gtk_editable_insert_text(editable, replace, strlen(replace), &pos_d);
                    gtk_editable_select_region(editable, pos,
                            pos + mbstowcs(NULL, replace, strlen(replace)));
                }
/*
                else
                    gtk_editable_set_position(editable, pos);
                    ʤgtk+
*/
                gtk_text_thaw(GTK_TEXT(text));
            }
            g_free(str);
            return;
        }
        pos++;
        p += mblen(p, MB_CUR_MAX);
    }
    g_free(str);
}

void searchBackward(GtkWidget* text, const char* find_str, const char* replace)
{
    int pos;
    GtkEditable* editable = GTK_EDITABLE(text);
    if (editable->selection_start_pos != editable->selection_end_pos)
        pos = editable->selection_start_pos;
    else
        pos = gtk_editable_get_position(editable);
    TRACE("pos = %d\n", pos);

    char* const str = gtk_editable_get_chars(editable, 0, pos);
    if (!str || str[0] == 0) {
        g_free(str);
        return;
    }

    const char* p = str;
    int find_pos = -1;
    pos = 0;
    for (int i = mbstowcs(NULL, str, strlen(str)); i > 0; i--) {
        if (!strncmp(p, find_str, strlen(find_str)))
            find_pos = pos;
        pos++;
        p += mblen(p, MB_CUR_MAX);
    }

    if (find_pos != -1) {
        int len = mbstowcs(NULL, find_str, strlen(find_str));
        if (!replace) {
            gtk_editable_select_region(editable, find_pos, find_pos + len);
            // gtk_editable_set_position(GTK_EDITABLE(frame->text), find_pos);
        }
        else {
            gtk_text_freeze(GTK_TEXT(text));
            gtk_editable_delete_text(editable, find_pos, find_pos + len);
            if (replace[0] != '\0') {
                int pos_d = find_pos;
                gtk_editable_insert_text(editable, replace, strlen(replace), &pos_d);
                gtk_editable_select_region(editable, find_pos,
                        find_pos + mbstowcs(NULL, replace, strlen(replace)));
            }
/*
            else
                gtk_editable_set_position(editable, pos);
*/
            gtk_text_thaw(GTK_TEXT(text));
        }
    }
    g_free(str);
}

void
on_search_forward_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(button));
    SearchDlg* this_ = frame->searchDlg;

    const char* find_str = gtk_entry_get_text(GTK_ENTRY(this_->search_entry));
    if (!find_str || find_str[0] == 0)
        return;

    searchForward(frame->text, find_str, NULL);
}


void
on_search_backward_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(button));
    SearchDlg* this_ = frame->searchDlg;

    const char* find_str = gtk_entry_get_text(GTK_ENTRY(this_->search_entry));
    if (!find_str || find_str[0] == 0)
        return;

    searchBackward(frame->text, find_str, NULL);
}

////////////////////////////////////////////////////////////////////////
// ReplaceDlg

ReplaceDlg::ReplaceDlg()
{
    window = create_replace_window();
}

ReplaceDlg::~ReplaceDlg()
{
}

void
on_replace_cancel                      (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget* window = lookup_widget(GTK_WIDGET(button), "replace_window");
    gtk_widget_hide(window);
}

gboolean
on_replace_window_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    gtk_widget_hide(widget);
    return TRUE;
}


void
on_replace_window_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(widget);
    ReplaceDlg* this_ = frame->replaceDlg;

    this_->search_entry = lookup_widget(widget, "search_entry");
    this_->replace_entry = lookup_widget(widget, "replace_entry");
    this_->confirm_check = lookup_widget(widget, "confirm_check");
}


void
on_replace_forward_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(button));
    ReplaceDlg* this_ = frame->replaceDlg;

    const char* find_str = gtk_entry_get_text(GTK_ENTRY(this_->search_entry));
    if (!find_str || find_str[0] == 0)
        return;
    const char* replace_str = gtk_entry_get_text(GTK_ENTRY(this_->replace_entry));
    if (!replace_str)
        return;

    searchForward(frame->text, find_str, replace_str);
}


void
on_replace_backward_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
    NotepadWindow* frame = findNotepadWindow(GTK_WIDGET(button));
    ReplaceDlg* this_ = frame->replaceDlg;

    const char* find_str = gtk_entry_get_text(GTK_ENTRY(this_->search_entry));
    if (!find_str || find_str[0] == 0)
        return;
    const char* replace_str = gtk_entry_get_text(GTK_ENTRY(this_->replace_entry));
    if (!replace_str)
        return;

    searchBackward(frame->text, find_str, replace_str);
}

////////////////////////////////////////////////////////////////////////


