// mini-notepad
// Copyright (c) 1998-1999 Hisashi HORIKAWA. All rights reserved.

#include <gtk/gtk.h>
#include <document.h>
#include <misc.h>

////////////////////////////////////////////////////////////////////////
// NotepadDoc

class NotepadWindow;
class NotepadDoc: public Document
{
    NotepadWindow* frame;
public:
    NotepadDoc(NotepadWindow* );
    virtual ~NotepadDoc();
    virtual bool save();
};

////////////////////////////////////////////////////////////////////////
// NotepadWindow

struct Heading
{
    string name;
    int pos;        // ʸñ̡h_startľؤ
};

typedef ptr_vector<Heading*> Headings;

class SearchDlg;
class ReplaceDlg;
class NotepadWindow: public Controller
{
public:
    GtkWidget* window;
    GtkWidget* text;
    GtkWidget* h_tree;
    SearchDlg* searchDlg;
    ReplaceDlg* replaceDlg;
    NotepadDoc* doc;
    Headings headings;

    NotepadWindow();
    virtual ~NotepadWindow();
    virtual void updateTitle(const string& );
    void updateHeadingTree();
    Headings::iterator insertHeading(Heading* h);

    static void onSearchMenuMap(GtkWidget* widget, NotepadWindow* this_);
    static void onWindowMenuMap(GtkWidget* widget, NotepadWindow* this_);
    static void onWindowWindow(GtkMenuItem* menuitem, NotepadWindow* target);
};

void
on_notepad_window_realize              (GtkWidget       *widget,
                                        gpointer         user_data);

gboolean
on_notepad_window_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data);

void
on_notepad_window_destroy              (GtkObject       *object,
                                        gpointer         user_data);

void
on_text_changed                        (GtkEditable     *editable,
                                        gpointer         user_data);

////////////////////////////////////////////////////////////////////////

typedef ptr_set<NotepadWindow*, less<NotepadWindow*> > Frames;
extern Frames frames;

extern bool loadFile(NotepadWindow* frame, const string& filename);

void
on_file_new_window_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_file_new_reuse_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_file_open_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_file_save_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_file_save_as_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_file_close_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_edit_cut_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_edit_copy_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_edit_paste_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_edit_clear_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_edit_select_all_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_insert_date_time_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_search_search_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_search_replace_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_help_about_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

////////////////////////////////////////////////////////////////////////
// SearchDlg

class SearchDlg
{
public:
    GtkWidget* window;
    GtkWidget* search_entry;

    SearchDlg();
    virtual ~SearchDlg();
};

gboolean
on_search_window_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data);

void
on_search_window_realize               (GtkWidget       *widget,
                                        gpointer         user_data);

void
on_search_forward_clicked              (GtkButton       *button,
                                        gpointer         user_data);

void
on_search_backward_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_search_cancel                       (GtkButton       *button,
                                        gpointer         user_data);

////////////////////////////////////////////////////////////////////////
// ReplaceDlg

class ReplaceDlg
{
public:
    GtkWidget* window;
    GtkWidget* search_entry;
    GtkWidget* replace_entry;
    GtkWidget* confirm_check;

    ReplaceDlg();
    virtual ~ReplaceDlg();
};

gboolean
on_replace_window_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data);

void
on_replace_window_realize              (GtkWidget       *widget,
                                        gpointer         user_data);

void
on_replace_forward_clicked             (GtkButton       *button,
                                        gpointer         user_data);

void
on_replace_backward_clicked            (GtkButton       *button,
                                        gpointer         user_data);

void
on_replace_cancel                      (GtkButton       *button,
                                        gpointer         user_data);


void
on_text_insert_text                    (GtkEditable     *editable,
                                        gchar           *new_text,
                                        gint             new_text_length,
                                        gint            *position,
                                        gpointer         user_data);

void
on_text_delete_text                    (GtkEditable     *editable,
                                        gint             start_pos,
                                        gint             end_pos,
                                        gpointer         user_data);

void
on_h_tree_tree_select_row              (GtkCTree        *ctree,
                                        GList           *node,
                                        gint             column,
                                        gpointer         user_data);

void
on_h_tree_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data);
